/***************************************************************************
 *     Copyright 2001  Broadcom Corporation
 *     All Rights Reserved
 *     No portions of this material may be reproduced in any form without the
 *     written permission of:
 *              Broadcom Corporation
 *              16251 Laguna Canyon Road
 *              Irvine, California  92618
 *     All information contained in this document is Broadcom Corporation
 *     company private, proprietary, and trade secret.
 *
 ****************************************************************************
 *
 *     Filename:  subkeyStorage.c
 *     Creation Date: 20 august 2001 (v0.00)
 *     VSS Info:
 *         $Revision: 1.1 $
 *         $Date: 2002/08/21 17:54:12 $
 *
 ****************************************************************************
 *     Description:
 *
 *       Implementation of subkey storage and retreival
 *
 *     Notes:
 *
 ****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "SecLib.h"
#include "subkeyStore.h"

#ifdef DEMO
#define GLOBAL_CFG_MAX_ENDPT   8
#else
#include <globalCfg.h>
#endif


#define MAX_SUBKEY_STORAGE_ENTRIES    GLOBAL_CFG_MAX_ENDPT

/* structs using in table to map info to NVRAM */
typedef struct
{
   unsigned char   t_hash[MD5_SIZE];          /* hash of cms fqdn */
   unsigned char   subkey[SUBKEY_LENGTH];
   unsigned long  sec_param_lifetime;
   unsigned long   grace_period;
} subkey_storage_info;





static subkey_storage_info subkey_storage_table[MAX_SUBKEY_STORAGE_ENTRIES];



/*
******************************************************************************
** FUNCTION:  secStoreSubkeyInfo
**
** PURPOSE:   store subkey for expiry, endpoint sharing etc.
**
** PARAMETERS:
**    1. subkey value
**    2. subkey length
**    3. fqdn of call agent
**    4. security parameter lifetime - number of seconds fron now.
**    5. grace period - in seconds
**
**
** RETURNS:
**
** NOTE:
**
******************************************************************************
*/
void secStoreSubkeyInfo(
      unsigned char   *subkey,
      unsigned char   *fqdn,
      unsigned long   sec_params_lifetime,
      unsigned long   grace_period)
{
   int             i;
   unsigned long   tmp_len;
   int             tmp_time;
   unsigned char   lfqdn[255];
   char            *p;

   strcpy(lfqdn, fqdn);
   p = lfqdn;
   while(*p)
   {
      *p++ = (char)tolower((int)*p);
   }

   /* remove if there is an existing entry */
   secRemoveSubkeyStored(lfqdn);

   /* find an open entry in the table */
   for (i=0;i<MAX_SUBKEY_STORAGE_ENTRIES;i++)
      if ( subkey_storage_table[i].t_hash[0] == '\0')
         break;
   if ( i == MAX_SUBKEY_STORAGE_ENTRIES )
   {
      printf("*SEC PANIC* no more subkey storage table entries.\n");
      return;
   }

   SecLibGetCryptoIface()->secCryptoCalcHash(HASH_MD5_TYPE, lfqdn, strlen(lfqdn),
         NULL, 0, (unsigned char *)(subkey_storage_table[i].t_hash), &tmp_len);

   /* Add current time to lifetime */
   time(&tmp_time);
   subkey_storage_table[i].sec_param_lifetime = sec_params_lifetime + tmp_time;
   subkey_storage_table[i].grace_period = grace_period;
   memcpy(subkey_storage_table[i].subkey, subkey, SUBKEY_LENGTH);
}


/*
******************************************************************************
** FUNCTION:  secRetreiveSubkeyInfo
**
** PURPOSE:   retreive subkey for expiry, endpoint sharing etc.
**
** PARAMETERS:
**    1. subkey value
**    2. subkey length
**    3. fqdn of call agent
**    4. security parameter lifetime - number of seconds fron now.
**    5. grace period - in seconds
**
**
** RETURNS:
**    1 if found,
**    0 otherwise.
**
** NOTE:
**     Caller responsible for allocating space for subkey
**
******************************************************************************
*/
unsigned long secRetreiveSubkeyInfo(
      unsigned char   *subkey,
      unsigned char   *fqdn,
      unsigned long   *sec_params_lifetime,
      unsigned long   *grace_period,
      int             *status)
{
   int               i;
   unsigned long     tmp_len;
   unsigned char     tmp_hash[MD5_SIZE];
   int               tmp_time;
   unsigned char     lfqdn[255];
   char              *p;

   strcpy(lfqdn, fqdn);
   p = lfqdn;
   while(*p)
   {
      *p++ = (char)tolower((int)*p);
   }

   SecLibGetCryptoIface()->secCryptoCalcHash(HASH_MD5_TYPE, lfqdn, strlen(lfqdn),
         NULL, 0, tmp_hash, &tmp_len);

   for (i=0;i<MAX_SUBKEY_STORAGE_ENTRIES;i++)
   {
      if ( ! memcmp(subkey_storage_table[i].t_hash, tmp_hash, MD5_SIZE))
      {
         time(&tmp_time);
         if (tmp_time < (subkey_storage_table[i].sec_param_lifetime -
                  subkey_storage_table[i].grace_period))
         {
            *status = SUBKEY_VALID;
         }
         else if ( (tmp_time >=  (subkey_storage_table[i].sec_param_lifetime -
                  subkey_storage_table[i].grace_period)) &&
               (tmp_time <= subkey_storage_table[i].sec_param_lifetime))
         {
            *status = SUBKEY_WITHIN_GRACE_PERIOD;
         }
         else
         {
            *status = SUBKEY_EXPIRED;
         }
         break;
      }
   }

   if ( i == MAX_SUBKEY_STORAGE_ENTRIES)
      return 0;

   if ( (*status == SUBKEY_VALID) || (*status == SUBKEY_WITHIN_GRACE_PERIOD))
   {
      memcpy(subkey, subkey_storage_table[i].subkey, SUBKEY_LENGTH);
      *sec_params_lifetime = subkey_storage_table[i].sec_param_lifetime;
      *grace_period = subkey_storage_table[i].grace_period;
      return *status;
   }
   else
   {
      secRemoveSubkeyStored(lfqdn);
      return SUBKEY_EXPIRED;
   }
}


/*
******************************************************************************
** FUNCTION:  secRemoveSubkeyStored
**
** PURPOSE:   remove subkey for the fqdn entry
**
** PARAMETERS:
**    1. fqdn of call agent
**
**
** RETURNS:
**
** NOTE:
**
******************************************************************************
*/
unsigned long secRemoveSubkeyStored(
      unsigned char   *fqdn)
{
   int               i;
   unsigned long     tmp_len;
   unsigned char     tmp_hash[MD5_SIZE];
   unsigned char     lfqdn[255];
   char              *p;

   strcpy(lfqdn, fqdn);
   p = lfqdn;
   while(*p)
   {
      *p++ = (char)tolower((int)*p);
   }

   SecLibGetCryptoIface()->secCryptoCalcHash(HASH_MD5_TYPE, lfqdn, strlen(lfqdn),
         NULL, 0, tmp_hash, &tmp_len);

   for (i=0;i<MAX_SUBKEY_STORAGE_ENTRIES;i++)
   {
      if ( ! memcmp(subkey_storage_table[i].t_hash, tmp_hash, MD5_SIZE))
      {
         subkey_storage_table[i].t_hash[0] = '\0';
         subkey_storage_table[i].subkey[0] = '\0';
         subkey_storage_table[i].sec_param_lifetime = 0;
         subkey_storage_table[i].grace_period = 0;
         return 1;
      }
   }
   return 0;
}
