/***************************************************************************
 *     Copyright 2000  Broadcom Corporation
 *     All Rights Reserved
 *     No portions of this material may be reproduced in any form without the
 *     written permission of:
 *              Broadcom Corporation
 *              16251 Laguna Canyon Road
 *              Irvine, California  92618
 *     All information contained in this document is Broadcom Corporation
 *     company private, proprietary, and trade secret.
 *
 ****************************************************************************
 *
 *     Filename:  SecLib.c
 *     Creation Date: 06 September 2000 (v0.00)
 *     VSS Info:
 *         $Revision: 1.4 $
 *         $Date: 2004/03/16 21:29:05 $
 *
 ****************************************************************************
 *     Description:
 *
 *       This file contains implementations of methods required by
 *       CablexChange Security purposes
 *
 *     Notes:
 *        OpenSSL package is used widely underneath.
 *
 ****************************************************************************/
/*
  6-6-02 Added OpenSSL includes
*/
/*
*****************************************************************************
* INCLUDE FILES
*****************************************************************************
*/

#ifdef WIN32
#define SECLIB_DLL
#define LINKDLL __declspec( dllexport)
#endif

/* system includes */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>


/* OpenSSL includes */
#include "crypto.h"
#include "dh.h"
#include "rand.h"
#include "asn1.h"
#include "x509.h"
#include "bio.h"
#include "x509v3.h"
#include "pem.h"
#include "rsa.h"
#include "bn.h"
#include "objects.h"
#include "apps.h"
#include "err.h"
#include "pem.h"
#include "sha.h"

/* library header file */
#include "SecLib.h"



/*
*****************************************************************************
* STATIC DECLARATIONS
*****************************************************************************
*/



static unsigned char *pErrBuf1;     /* error buffer to be loaded in callback function */
static int errBufLength;            /* length of the error buffer in validate function */
/* return code for validate function - assigned in callback function */
static int errIndicator;  

static unsigned long secCertGetPublicKey( 
      unsigned char     *pCertBuf,
      unsigned int      pCertBufLen,
      RSA_key           *key,
      unsigned int      *pPubKeyId );
static unsigned long secCertValidate( 
      unsigned char    *pCertBuf,
      unsigned int      pCertBufLen,
      unsigned int      pCertCALen,
      unsigned char     *pCertCA,
      unsigned char     *pErrBuf,
      int               errBufLen );
static unsigned long secCertGetSerNum( 
      unsigned char     *pCertBuf,
      unsigned int      pCertBufLen,
      unsigned char     *pSerialNumber,
      unsigned long     *pSerialNumnerLen);
static unsigned long secCertGetSigAlgId( 
      unsigned char     *pCertBuf, 
      unsigned int      pCertBufLen, 
      unsigned long     *pAlgId );
static unsigned long secCertGetNotAfterDate( 
      unsigned char     *pCertBuf,
      unsigned int      pCertBufLen,
      unsigned char     *pNotAfterBuf,
      unsigned int      *pNotAfterBufLen );
static unsigned long secCertGetNotBeforeDate(
      unsigned char     *pCertBuf,
      unsigned int      pCertBufLen,
      unsigned char     *pNotBeforeBuf,
      unsigned int      *pNotBeforeBufLen );
static unsigned long secCertGetSubjectName( 
      unsigned char     *pCertBuf,
      unsigned long     pCertBufLen,
      unsigned char     *pSubjName,
      unsigned long     *subjNameLen );
static unsigned long secCertGetSubjectNameOS( 
      unsigned char     *pCertBuf,
      unsigned long     pCertBufLen,
      unsigned char     *pSubjName,
      unsigned long     *subjNameLen );
static unsigned long secCertGetIssuerName( 
      unsigned char     *pCertBuf,
      unsigned long     pCertBufLen,
      unsigned char     **pIssuerName,
      unsigned long     *issuerNameLen );
static unsigned long secCertGetIssuerNameOS( 
      unsigned char     *pCertBuf,
      unsigned long     pCertBufLen,
      unsigned char     **pIssuerName,
      unsigned long     *issuerNameLen );
static unsigned long secCertGetOrgName(
      unsigned char    *subjectName,
      unsigned long    subjectNameLength,
      unsigned char    *orgName,
      unsigned long    *orgNameLength);
static unsigned long secCertGetSubjectAltNameExt(
      unsigned char     *pCertBuf,
      unsigned long     pCertBufLen,
      unsigned char     *subjectAltNameOid,
      unsigned long     *subjectAltNameOidLength,
      unsigned char     *subjectAltName,
      unsigned long     *subjectAltNameLength);



/* assign functions to pointers */
static SEC_CERT_IFACE   secCertIface =
{
   secCertGetPublicKey ,
   secCertValidate ,
   secCertGetSerNum ,
   secCertGetSigAlgId ,
   secCertGetNotAfterDate ,
   secCertGetNotBeforeDate,
   secCertGetSubjectName,
   secCertGetSubjectNameOS,
   secCertGetIssuerName,
   secCertGetIssuerNameOS,
   secCertGetOrgName,
   secCertGetSubjectAltNameExt,
};


/* callback function used in cert verification */
static int MS_CALLBACK cb(int ok, X509_STORE_CTX *ctx);
/* decode method for certificate issues */
static X509* decodeDERCert( unsigned char *pCertBuf, unsigned int bufferLength );

static int v_verbose;


/*
*****************************************************************************
**
** FUNCTION:   Init_secCert
**
** PURPOSE:    Initializes the Cert Part of the SecLib**
**
** PARAMETERS: NONE
**
** RETURNS:    VOID
**
** NOTES:      This call is direct, no need for redirection in the
**             "SEC_CRYPTO_IFACE".
*****************************************************************************
*/
void Init_secCert(void)
{
    v_verbose = 0;
    errIndicator = 0;
    pErrBuf1 = NULL;
}

/* This function is not called by anyone so far.
   It's just a place holder for a future extension.
*/
void Deinit_secCert(void)
{

}

/*
*****************************************************************************
**
**  Function Name: SecLibGetCertIface
**
**  PURPOSE:   return iface struct containing pointers to seclib functions
**
**  PARAMETERS:
**
**  RETURNS:
**
**  NOTES:
*****************************************************************************
*/
SEC_CERT_IFACE*   SecLibGetCertIface(void)
{
   return &secCertIface;
}


/*
*****************************************************************************
**
**  Function Name: secCertGetPublicKey
**
**  PURPOSE:
**     Retrieve public key and its length. Key buffer contains key itself in HEX
**
**  PARAMETERS:
**      1. pCertBuf - pointer to the certicate buffer.
**      2. pCertBufLen - length of the certificate buffer
**      3. key - pointer to the internal repr. of RSA key, used as an output parameter
**      4. pPubKeyId - key identifier
**
**  RETURNS:
**     succes or error code - unsigned long
**
**  NOTES:
**
*****************************************************************************
*/
static unsigned long secCertGetPublicKey( 
      unsigned char     *pCertBuf,
      unsigned int      pCertBufLen,
      RSA_key           *key,
      unsigned int      *pPubKeyId )
{
   X509 *x509p;           /* certificate */
   EVP_PKEY *publicKeyp;  /* key envelope */
   char temp[500];               /* local helper */


   x509p = decodeDERCert( pCertBuf, pCertBufLen );

   if ( !x509p )
   {
      return MEM_ALLOC_ERROR;
   }

   /* is key there ? */
   if ( !(publicKeyp = X509_get_pubkey( x509p )) )
   {
      return BAD_DATA;
   }

   /* check for key type */

   if ( publicKeyp->type == EVP_PKEY_RSA )
   {
      /* key ID */
      *pPubKeyId = EVP_PKEY_RSA;

      /* key length in bits */
      key->keyLenBits = BN_num_bits(publicKeyp->pkey.rsa->n);

      /* get the public key in binary */
      key->modBufLen = BN_bn2bin( publicKeyp->pkey.rsa->n, temp );
      key->mod = (unsigned char*)malloc( key->modBufLen );
      memcpy( key->mod, temp, key->modBufLen );

      key->expBufLen = BN_bn2bin( publicKeyp->pkey.rsa->e, temp );
      key->exp = (unsigned char*)malloc( key->expBufLen );
      memcpy( key->exp, temp, key->expBufLen );

      /* free the certificate storage */
      X509_free( x509p );

      EVP_PKEY_free( publicKeyp );

      ERR_remove_state( 0 );

      return OK;
   }

#if 0
   else if ( publicKeyp->type == EVP_PKEY_DSA )
   {
      /* key ID */
      *pPubKeyId = EVP_PKEY_DSA;

      /* key length in bits */
      *pPubKeyLenBits = BN_num_bits(publicKeyp->pkey.dsa->p);

      /* get the public key in binary */
      bytesWritten = BN_bn2bin( publicKeyp->pkey.dsa->priv_key, temp );

      /* check for input buffer length */
      if ( *pPubKeyBufLen <= bytesWritten )
      {
         /* buffer too short, return error */
         return BUF_SHORT;
      }

      /* buffer is ok, update the length and copy the key to the proper place */
      *pPubKeyBufLen = bytesWritten;
      memcpy( pPubKeyBuf, temp, *pPubKeyBufLen );

      /* free the certificate storage */
      X509_free( x509p );
      EVP_PKEY_free( publicKeyp );
      ERR_remove_state( 0 );

      return OK;
   }
#endif

   /* free the certificate storage */
   X509_free( x509p );
   EVP_PKEY_free( publicKeyp );
   ERR_remove_state( 0 );
   /* we have two types of keys defined RSA and DSA, others won't do */
   return BAD_DATA;
}



/*
*****************************************************************************
**
**  Function Name: secCertValidate
**
**  PURPOSE:
**     Verify certificate against provided CA certificate.
**
**  PARAMETERS:
**      1. pCertBuf - pointer to the certicate buffer.
**      2. pCertBufLen - length of the certificate buffer
**      3. pCertCALen - length of the CA certificate buffer
**      4. pCertCA -  pointer to CA certificate buffer
**      5. pErrBuf - pointer to the error buffer, used as an output parameter
**      6. errBufLen - length of error buffer
**
**  RETURNS:
**     succes or error code - unsigned long
**
**  NOTES:
**
*****************************************************************************
*/
static unsigned long secCertValidate( 
      unsigned char    *pCertBuf,
      unsigned int      pCertBufLen,
      unsigned int      pCertCALen,
      unsigned char     *pCertCA,
      unsigned char     *pErrBuf,
      int               errBufLen )
{

	X509_STORE *cert_ctx;
	X509_LOOKUP *lookup;
   X509 *x509p;
   X509 *x509CA;
   X509_STORE_CTX *csc;
   unsigned int currentCertLen, totalLen = 0;
   unsigned int temp;

   /*printf("secCertValidate  *pCertBuf<%p>, pCertBufLen<%d>, pCertCALen<%d>, *pCertCA<%p>, *pErrBuf<%p>, errBufLen<%d> \n",
                            pCertBuf,       pCertBufLen,     pCertCALen,     pCertCA,      pErrBuf,      errBufLen );
   */

   errIndicator = 0;

   x509p = decodeDERCert( pCertBuf, pCertBufLen );

   if ( !x509p )
   {
      return MEM_ALLOC_ERROR;
   }

   /* set static to point to output buffer */
   pErrBuf1 = pErrBuf;

   /* assign value to static varible */
   errBufLength = errBufLen;

   /* this piece of code is stripped down from OpenSSL apps/verify.c file */
   cert_ctx=X509_STORE_new();

   if (cert_ctx == NULL) return BAD_DATA;

	X509_STORE_set_verify_cb_func(cert_ctx,cb);

	/* ERR_load_crypto_strings(); */


#ifdef TARGETOS_eCos
    //if we are eCos this do_pipe_sig causes a crash.
    //so just call the remaining pieces of apps_startup.
      
    //do_pipe_sig();                    
    CRYPTO_malloc_init();             
    SSLeay_add_all_algorithms();      
#else
   apps_startup();
#endif

   /* run apps_startup only once - controlled by static variable appsInd */
   /*if ( !appsInd )
   {
      apps_startup();
      appsInd = 1;
   }*/

   lookup=X509_STORE_add_lookup(cert_ctx,X509_LOOKUP_file() );

	if (lookup == NULL) return BAD_DATA;

   /* start parsing out the CA buffer */
   while (1)
   {
      if ( *(pCertCA+1) == 0x82 )
      {
         temp = ( *(pCertCA+2) << 8 ) | ( *(pCertCA+3) );
         currentCertLen = temp + 4;
      }
      else
      {
         temp = *(pCertCA+2);
         currentCertLen = temp +3;
      }

      totalLen +=currentCertLen;

      if ( totalLen > pCertCALen )
      {
         return BAD_DATA;
      }

      x509CA = decodeDERCert( pCertCA, currentCertLen );

      if ( !x509CA )
      {
         return MEM_ALLOC_ERROR;
      }

      /* add CA cert to lookup */
      X509_STORE_add_cert( lookup->store_ctx, x509CA );

      /* free the certificate storage */
      X509_free( x509CA );

      pCertCA = pCertCA+currentCertLen;

      if ( totalLen == pCertCALen )
      {
         break;
      }

   }

	ERR_clear_error();

   csc = X509_STORE_CTX_new();

   /* add certificate to be verified */
   X509_STORE_CTX_init(csc,cert_ctx,x509p,NULL);

   /* set purpose */
   X509_STORE_CTX_set_purpose(csc, 7);

   /* verify the certificate */
   X509_verify_cert(csc);

   /* free dynamic memory */
   X509_STORE_CTX_free(csc);

	/* return success or failure */
   if (cert_ctx != NULL) X509_STORE_free(cert_ctx);

   /* free the certificate storage */
   X509_free( x509p );

   /* clean all ciphers&digests set by apps_startup() */
   EVP_cleanup();

   ERR_remove_state( 0 );
   /* errIndicator contains appropriate return code - assigned in callback function */
   return errIndicator;
}
           
/*
******************************************************************************
**  Function Name: secCertGetSerNum
**
**  PURPOSE:
**     Get certificate serial number
**
**  PARAMETERS:
**      1. pCertBuf - pointer to the certicate buffer.
**      2. pCertBufLen - length of the certificate buffer
**      3. pSerialNumber -  certificate serial number, used as an output parameter
**
**  RETURNS:
**     succes or error code - unsigned long
**
**  NOTES:
**
*****************************************************************************
*/
static unsigned long secCertGetSerNum( 
      unsigned char     *pCertBuf,
      unsigned int      pCertBufLen,
      unsigned char     *pSerialNumber,
      unsigned long     *pSerialNumberLen)
{
    X509 *x509p; /* certificate */


    x509p = decodeDERCert( pCertBuf, pCertBufLen );

    if ( !x509p )
    {
       return MEM_ALLOC_ERROR;
    }

    /* serial number is valid ? */
    if ( !(&x509p->cert_info->serialNumber) )
    {
       return BAD_DATA;
    }

#if 0
    /* copy serial number value to the input buffer and return succes */
    *pSerialNumber = (unsigned long)ASN1_INTEGER_get(*(&x509p->cert_info->serialNumber));
#endif
    *pSerialNumberLen = x509p->cert_info->serialNumber->length;
    memcpy(pSerialNumber, x509p->cert_info->serialNumber->data, *pSerialNumberLen);

    /* free the certificate storage */
    X509_free( x509p );

    ERR_remove_state( 0 );
    return OK;
}


/*
******************************************************************************
**  Function Name: secCertGetSigAlgId
**
**  PURPOSE:
**     Get certificate signature algorithm ID
**
**  PARAMETERS:
**      1. pCertBuf - pointer to the certicate buffer.
**      2. pCertBufLen - length of the certificate buffer
**      3. pAlgId -  certificate signature algorithm identifier, an output parameter
**
**  RETURNS:
**     succes or error code - unsigned long
**
**  NOTES:
**
******************************************************************************
*/
static unsigned long secCertGetSigAlgId( 
      unsigned char     *pCertBuf, 
      unsigned int      pCertBufLen, 
      unsigned long     *pAlgId )
{
   X509 *x509p;  /* certificate */


   x509p = decodeDERCert( pCertBuf, pCertBufLen );

   if ( !x509p )
   {
      return MEM_ALLOC_ERROR;
   }

   /* signature alg id exists ? */
   if ( !(&x509p->cert_info->signature->algorithm) )
   {
      return BAD_DATA;
   }

   /* get the sig ID, copy it to the buffer and return success */
   *pAlgId = OBJ_obj2nid(*(&x509p->cert_info->signature->algorithm));

   /* free the certificate storage */
   X509_free( x509p );

   ERR_remove_state( 0 );
   return OK;
}


/*
*****************************************************************************
**  Function Name: secCertGetNotAfterDate
**
**  PURPOSE:
**      Get certificate not after date
**
**  PARAMETERS:
**      1. pCertBuf - pointer to the certicate buffer.
**      2. pCertBufLen - length of the certificate buffer
**      3. pNotAfterBuf - pointer to the not after date buffer, output paramete
**      4. pNotAfterBufLen - length of not after date buffer, I/O parameter
**
**  RETURNS:
**     succes or error code - unsigned long
**
**  NOTES:
**
******************************************************************************
*/
static unsigned long secCertGetNotAfterDate( 
      unsigned char     *pCertBuf,
      unsigned int      pCertBufLen,
      unsigned char     *pNotAfterBuf,
      unsigned int      *pNotAfterBufLen )
{
   X509 *x509p;  /* certificate */
   unsigned int temp;   /* local helper */



   x509p = decodeDERCert( pCertBuf, pCertBufLen );

   if ( !x509p )
   {
      return MEM_ALLOC_ERROR;
   }

   /* time is there ? */
   if ( !(&x509p->cert_info->validity->notAfter->data) )
   {
      return BAD_DATA;
   }


   /* get the length of time buffer */
   temp = strlen(*(&x509p->cert_info->validity->notAfter->data));

   /* check for input buffer length */
   if ( *pNotAfterBufLen < temp )
   {
      /* length is too short, return error */
      return BUF_SHORT;
   }

   /* copy the info to the buffers and return success */
   strncpy( pNotAfterBuf, *(&x509p->cert_info->validity->notAfter->data), temp );
   *pNotAfterBufLen = temp;

   /* free the certificate storage */
   X509_free( x509p );

   ERR_remove_state( 0 );
   return OK;
}


/*
*****************************************************************************
**  Function Name: secCertGetNotBeforeDate
**
**  PURPOSE:
**      Get certificate not before date
**
**  PARAMETERS:
**      1. pCertBuf - pointer to the certicate buffer.
**      2. pCertBufLen - length of the certificate buffer
**      3. pNotAfterBuf - pointer to the not before date buffer, output parameter
**      4. pNotAfterBufLen - length of not before date buffer, I/O parameter
**
**  RETURNS:
**     succes or error code - unsigned long
**
**  NOTES:
**
******************************************************************************
*/
static unsigned long secCertGetNotBeforeDate(
      unsigned char     *pCertBuf,
      unsigned int      pCertBufLen,
      unsigned char     *pNotBeforeBuf,
      unsigned int      *pNotBeforeBufLen )
{
   X509 *x509p;  /* certificate */
   unsigned int temp;   /* local helper */


   x509p = decodeDERCert( pCertBuf, pCertBufLen );

   if ( !x509p )
   {
      return MEM_ALLOC_ERROR;
   }

   /* time is there ? */
   if ( !(&x509p->cert_info->validity->notBefore->data) )
   {
      return BAD_DATA;
   }

   /* get the length of time buffer */
   temp = strlen(*(&x509p->cert_info->validity->notBefore->data));


   /* check for input buffer length */
   if ( *pNotBeforeBufLen < temp )
   {
      /* length is too short, return error */
      return BUF_SHORT;
   }

   /* copy the info to the buffers and return success */
   strncpy( pNotBeforeBuf, *(&x509p->cert_info->validity->notBefore->data), temp );
   *pNotBeforeBufLen = temp;

   /* free the certificate storage */
   X509_free( x509p );

   ERR_remove_state( 0 );
   return OK;
}

/*
*****************************************************************************
**  Function Name: secCertGetSubjectName
**
**  PURPOSE:
**      Get certificate subject name as a printable string
**
**  PARAMETERS:
**      1. pCertBuf - pointer to the certicate buffer.
**      2. pCertBufLen - length of the certificate buffer
**      3. pSubjName - pointer to the subject name buffer, output parameter
**      4. subjNameLen - length of the subject name buffer, output parameter
**
**  RETURNS:
**     succes or error code - unsigned long
**
**  NOTES:
**
******************************************************************************
*/
static unsigned long secCertGetSubjectName( 
      unsigned char     *pCertBuf,
      unsigned long     pCertBufLen,
      unsigned char     *pSubjName,
      unsigned long     *subjNameLen )
{
   X509 *x509p;        /* certificate */
   char buf[256];             /* buffer to store name */


   /* decode cert from DER format */
   x509p = decodeDERCert( pCertBuf, pCertBufLen );

   if ( !x509p )
   {
      return MEM_ALLOC_ERROR;
   }

   /* subject name exists ? */
   if ( !x509p->cert_info->subject )
   {
      return BAD_DATA;
   }

   /* get the subject name */
   if ( !X509_NAME_oneline( X509_get_subject_name( x509p ), buf, 256 ) )
   {
      return BAD_DATA;
   }

   /* get the size of the buffer */
   *subjNameLen = strlen( buf );

   /* copy the subject name to the buffer */
   memcpy( pSubjName, buf, *subjNameLen );
   pSubjName[*subjNameLen] = '\0';

   /* free the certificate storage */
   X509_free( x509p );

   ERR_remove_state( 0 );
   /* return success */
   return OK;
   }

/*
*****************************************************************************
**  Function Name: secCertGetSubjectNameOS
**
**  PURPOSE:
**      Get certificate subject name ASN1 encoded as an octet string
**
**  PARAMETERS:
**      1. pCertBuf - pointer to the certicate buffer.
**      2. pCertBufLen - length of the certificate buffer
**      3. pSubjName - pointer to the subject name buffer, output parameter
**      4. subjNameLen - length of the subject name buffer, output parameter
**
**  RETURNS:
**     succes or error code - unsigned long
**
**  NOTES:
**
******************************************************************************
*/
static unsigned long secCertGetSubjectNameOS( 
      unsigned char     *pCertBuf,
      unsigned long     pCertBufLen,
      unsigned char     *pSubjName,
      unsigned long     *subjNameLen )
{
   X509 *x509p;               /* certificate */
   X509_NAME *name;           /* subject name */


   /* decode cert from DER format */
   x509p = decodeDERCert( pCertBuf, pCertBufLen );

   if ( !x509p )
   {
      return MEM_ALLOC_ERROR;
   }

   /* subject name exists ? */
   if ( !x509p->cert_info->subject )
   {
      return BAD_DATA;
   }

   /* get the subject name */
   name = X509_get_subject_name( x509p );

   /* get the size of the buffer */
   *subjNameLen = name->bytes->length;

   /* copy the subject name to the buffer in ASN1 encoded form */
   memcpy( pSubjName, name->bytes->data, *subjNameLen );

   /* free the certificate storage */
   X509_free( x509p );

   ERR_remove_state( 0 );
   /* return success */
   return OK;
   }

/*
******************************************************************************
**  Function Name: secCertGetIssuerName
**
**  PURPOSE:
**      Get certificate issuer name ASN1 as a printable string
**
**  PARAMETERS:
**      1. pCertBuf - pointer to the certicate buffer.
**      2. pCertBufLen - length of the certificate buffer
**      3. pIssuerName - pointer to the subject name buffer, output parameter
**      4. issuerNameLen - length of the subject name buffer, output parameter
**
**  RETURNS:
**     succes or error code - unsigned long
**
**  NOTES:
**
******************************************************************************
*/
static unsigned long secCertGetIssuerName( 
      unsigned char     *pCertBuf,
      unsigned long     pCertBufLen,
      unsigned char     **pIssuerName,
      unsigned long     *issuerNameLen )
{
   X509 *x509p;      /* certificate */
   char buf[256];    /* buffer to store the name */


   /* decode the cert from DER format */
   x509p = decodeDERCert( pCertBuf, pCertBufLen );

   if ( !x509p )
   {
      return MEM_ALLOC_ERROR;
   }

   /* issuer name exists ? */
   if ( !x509p->cert_info->issuer )
   {
      return BAD_DATA;
   }

   /* get the issuer name */
   if ( !X509_NAME_oneline( X509_get_issuer_name( x509p ), buf, 256 ) )
   {
      return BAD_DATA;
   }

   /* get the size of the buffer */
   *issuerNameLen = strlen( buf );

   /* allocate space for output buffer */
   *pIssuerName = (unsigned char*)malloc( *issuerNameLen );

   /* copy the subject name to the proper place */
   memcpy( *pIssuerName, buf, *issuerNameLen );

   X509_free( x509p );

   ERR_remove_state( 0 );
   /* return success */
   return OK;

}

/*
******************************************************************************
**  Function Name: secCertGetIssuerNameOS
**
**  PURPOSE:
**      Get certificate issuer name ASN1 encoded as an octet string
**
**  PARAMETERS:
**      1. pCertBuf - pointer to the certicate buffer.
**      2. pCertBufLen - length of the certificate buffer
**      3. pIssuerName - pointer to the subject name buffer, output parameter
**      4. issuerNameLen - length of the subject name buffer, output parameter
**
**  RETURNS:
**     succes or error code - unsigned long
**
**  NOTES:
**
*****************************************************************************
*/
static unsigned long secCertGetIssuerNameOS( 
      unsigned char     *pCertBuf,
      unsigned long     pCertBufLen,
      unsigned char     **pIssuerName,
      unsigned long     *issuerNameLen )
{
   X509 *x509p;     /* certificate */
   X509_NAME *name;        /* subject name */

   /* decode the cert from DER format */
   x509p = decodeDERCert( pCertBuf, pCertBufLen );

   if ( !x509p )
   {
      return MEM_ALLOC_ERROR;
   }

   /* issuer name exists ? */
   if ( !x509p->cert_info->issuer )
   {
      return BAD_DATA;
   }

   /* get the issuer name */
   name = X509_get_issuer_name( x509p );

   /* get the size of the buffer */
   *issuerNameLen = name->bytes->length;

   /* allocate space for output buffer */
   *pIssuerName = (unsigned char*)malloc( *issuerNameLen );

   /* copy the subject name to the proper place */
   memcpy( *pIssuerName, name->bytes->data, *issuerNameLen );

   /* free the certificate storage */
   X509_free( x509p );

   ERR_remove_state( 0 );

   /* return success */
   return OK;

}

/*
******************************************************************************
**  Function Name: cb
**
**  PURPOSE:
**     Callback function used in certificate verification - stored error/warnings if any
**
**  PARAMETERS:
**      1. ok - error indicator passed to function by some other method
**      2. ctx - certificate storage buffer, contains all info about certificat
**
**  RETURNS:
**     succes or error code
**
**  NOTES:
**
*****************************************************************************
*/
static int MS_CALLBACK cb(int ok, X509_STORE_CTX *ctx)
{
	char buf[256];
   char temp[256];

	if (!ok)
	{
		errIndicator = SELF_SIGN_OR_PATH;

      X509_NAME_oneline(
				X509_get_subject_name(ctx->current_cert),buf,256);

      /* put the warning/error in the buffer */
      sprintf( temp,"%s %s", buf, X509_verify_cert_error_string(ctx->error) );
      strncpy( pErrBuf1, temp, errBufLength );


		if (ctx->error == X509_V_ERR_CERT_HAS_EXPIRED)
      {
         ok=1;
         errIndicator = CERT_EXPIRED;
      }

		/* since we are just checking the certificates, it is
		 * ok if they are self signed. But we should still warn
		 * the user.
 		 */
		if (ctx->error == X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT) ok=1;
		/* Continue after extension errors too */

      if (ctx->error == X509_V_ERR_INVALID_CA)
      {
         ok=1;
         errIndicator = CA_INVALID;
      }

		if (ctx->error == X509_V_ERR_PATH_LENGTH_EXCEEDED) ok=1;

	}
	if (!v_verbose)
		ERR_clear_error();
	return(ok);
}

/*
******************************************************************************
**  Function Name: decodeDERCert
**
**  PURPOSE:
**     To decode certificate content from DER format
**
**  PARAMETERS:
**      1. pCertBuf - pointer to the certicate buffer.
**      2. bufferLength - length of the certificate buffer
**
**  RETURNS:
**     Pointer to the certificate data structure - decoded
**
**  NOTES:
**
******************************************************************************
*/
static X509* decodeDERCert( unsigned char *pCertBuf, unsigned int bufferLength )
{

   X509 *x509;   /* certificate to be returned */
   BIO *in;             /* intermediate buffer */

   /* allocate BIO buffer with appropriate method */
   in = BIO_new( BIO_s_mem());

   if ( !in )
   {
      return NULL;
   }

   /* write to BIO buffer from certificate buffer */
   in->method->bwrite( in, pCertBuf, bufferLength );

   /* decode DER content to pure x509 structure */
   x509 = d2i_X509_bio( in, NULL );

   /* free BIO buffer */
   BIO_free( in );

   /* return decoded certificate */
   return x509;
}


/*
******************************************************************************
**  Function Name: secCertGetOrgName
**
**  PURPOSE:   Get the orgName out of subjectName of a certificate (O=)
**
**  PARAMETERS:
**      1. subjectName - conatins all name components
**      2. subjectNameLength
**      3. orgName - returned orgName (O=)
**      4. orgNameLength - returned length of orgname
**
**  RETURNS:
**      0 if success, 1 if failure
**
**  NOTES:
**
******************************************************************************
*/
static unsigned long secCertGetOrgName(
      unsigned char    *subjectName,
      unsigned long    subjectNameLength,
      unsigned char    *orgName,
      unsigned long    *orgNameLength)
{
   char   *p;
   char   *q;

   /* Look for "O=" */
   p = strstr(subjectName, "O=");
   if ( !p )
   {
      printf("NO OrgName found in subjectName of certificate ( %s ).\n", subjectName);
      return 1;
   }

   /* pass O= and copy */
   p+=2;
   strcpy(orgName, p);

   /* name ends at "/OU=" */
   q = strstr(p, "/OU");
   if ( !q )
   {
      q = strstr(p, "/CN");
      if ( !q )
      {
         printf("BAD certificate Subject Name,can not parse orgName( %s ).\n", subjectName); 
         return 1;
      }
   }
   orgName[q - p] = '\0';
   *orgNameLength = strlen(orgName);
   return 0;
}


/*
******************************************************************************
**  Function Name: secCertGetSubjectAltNameExt
**
**  PURPOSE:   Get the subjectAltName extension
**
**  PARAMETERS:
**      1. pCertBuf
**      2. pCertBufLen
**      3. subjectAltName
**      4. subjectAltNameLen
**
**  RETURNS:
**      0 if success, 1 if failure
**
**  NOTES:
**
******************************************************************************
*/
static unsigned long secCertGetSubjectAltNameExt(
      unsigned char     *pCertBuf,
      unsigned long     pCertBufLen,
      unsigned char     *subjectAltNameOid,
      unsigned long     *subjectAltNameOidLength,
      unsigned char     *subjectAltName,
      unsigned long     *subjectAltNameLength)
{

   int                         i;
   X509                        *x509p;
	STACK_OF(GENERAL_NAME)      *ialt;
	GENERAL_NAME                *gen;
	X509_EXTENSION              *ext;

   x509p = decodeDERCert( pCertBuf, pCertBufLen );
   if ( !x509p )
   {
      return MEM_ALLOC_ERROR;
   }

   i = X509_get_ext_by_NID(x509p, NID_subject_alt_name, -1);
	if(i < 0)
   {
      printf("ERROR Parsing Subject Alt Name Extension.");
      return MEM_ALLOC_ERROR;
   }
   if(!(ext = X509_get_ext(x509p, i)) ||
         !(ialt = X509V3_EXT_d2i(ext)) )
   {
      printf("ERROR Parsing Subject Alt Name Content.");
      return MEM_ALLOC_ERROR;
   }
   if ( sk_GENERAL_NAME_num(ialt) > 1 )
   {
      printf("ERROR Parsing Subject Alt Name Content, More then 1 name sequence.");
      return MEM_ALLOC_ERROR;
   }
   gen = sk_GENERAL_NAME_value(ialt, 0);
   strcpy(subjectAltName, gen->d.otherName->value->value.asn1_string->data);
   *subjectAltNameLength = gen->d.otherName->value->value.asn1_string->length;
   strcpy(subjectAltNameOid, gen->d.otherName->type_id->data);
   *subjectAltNameOidLength = gen->d.otherName->type_id->length;
	sk_GENERAL_NAME_free(ialt);

   return 0;
}


